@@ -9,17 +9,25 @@ import re
 import os
 from nose import with_setup
 from nose.plugins.skip import SkipTest
+from nose.tools import assert_raises
+from time import sleep
 
 from tqdm import tqdm
 from tqdm import trange
+from tqdm import TqdmDeprecationWarning
+from tqdm._tqdm import TMonitor
 
 try:
     from StringIO import StringIO
-except:
+except ImportError:
     from io import StringIO
 
 from io import IOBase  # to support unicode strings
 
+
+class DeprecationError(Exception):
+    pass
+
 # Ensure we can use `with closing(...) as ... :` syntax
 if getattr(StringIO, '__exit__', False) and \
    getattr(StringIO, '__enter__', False):
@@ -42,7 +50,7 @@ nt_and_no_colorama = False
 if os.name == 'nt':
     try:
         import colorama  # NOQA
-    except:
+    except ImportError:
         nt_and_no_colorama = True
 
 # Regex definitions
@@ -70,11 +78,23 @@ class DiscreteTimer(object):
         return self.t
 
 
+class FakeSleep(object):
+    '''Wait until the discrete timer reached the required time'''
+    def __init__(self, dtimer):
+        self.dtimer = dtimer
+
+    def sleep(self, t):
+        end = t + self.dtimer.t
+        while(self.dtimer.t < end):
+            sleep(0.0000001)  # sleep a bit to interrupt (instead of pass)
+
+
 def cpu_timify(t, timer=None):
     '''Force tqdm to use the specified timer instead of system-wide time()'''
     if timer is None:
         timer = DiscreteTimer()
     t._time = timer.time
+    t._sleep = timer.sleep
     t.start_t = t.last_print_t = t._time()
     return timer
 
@@ -277,10 +297,10 @@ def test_all_defaults():
             for _ in progressbar:
                 pass
     # restore stdout/stderr output for `nosetest` interface
-    try:
-        sys.stderr.write('\x1b[A')
-    except:
-        pass
+    # try:
+    #     sys.stderr.write('\x1b[A')
+    # except:
+    #     pass
     sys.stderr.write('\rTest default kwargs ... ')
 
 
@@ -298,7 +318,7 @@ def test_iterate_over_csv_rows():
         reader = csv.DictReader(test_csv_file,
                                 fieldnames=('row1', 'row2', 'row3'))
         with closing(StringIO()) as our_file:
-            for row in tqdm(reader, file=our_file):
+            for _ in tqdm(reader, file=our_file):
                 pass
 
 
@@ -823,12 +843,12 @@ def test_deprecated_nested():
     our_file = StringIO()
     try:
         tqdm(total=2, file=our_file, nested=True)
-    except DeprecationWarning as e:
-        if str(e) != ("nested is deprecated and automated.\nUse position"
-                      " instead for manual control"):
+    except TqdmDeprecationWarning:
+        if """`nested` is deprecated and automated.\
+ Use position instead for manual control.""" not in our_file.getvalue():
             raise
     else:
-        raise DeprecationWarning("Should not allow nested kwarg")
+        raise DeprecationError("Should not allow nested kwarg")
 
 
 @with_setup(pretest, posttest)
@@ -836,7 +856,7 @@ def test_bar_format():
     """ Test custom bar formatting """
     with closing(StringIO()) as our_file:
         bar_format = r'{l_bar}{bar}|{n_fmt}/{total_fmt}-{n}/{total}{percentage}{rate}{rate_fmt}{elapsed}{remaining}'  # NOQA
-        for i in trange(2, file=our_file, leave=True, bar_format=bar_format):
+        for _ in trange(2, file=our_file, leave=True, bar_format=bar_format):
             pass
         out = our_file.getvalue()
     assert "\r  0%|          |0/2-0/20.0None?it/s00:00?\r" in out
@@ -897,11 +917,11 @@ def test_position():
 
     # Test iteration-based tqdm positioning
     our_file = StringIO()
-    for i in trange(2, file=our_file, miniters=1, mininterval=0,
+    for _ in trange(2, file=our_file, miniters=1, mininterval=0,
                     maxinterval=0, desc='pos0 bar', position=0):
-        for j in trange(2, file=our_file, miniters=1, mininterval=0,
+        for _ in trange(2, file=our_file, miniters=1, mininterval=0,
                         maxinterval=0, desc='pos1 bar', position=1):
-            for k in trange(2, file=our_file, miniters=1, mininterval=0,
+            for _ in trange(2, file=our_file, miniters=1, mininterval=0,
                             maxinterval=0, desc='pos2 bar', position=2):
                 pass
     our_file.seek(0)
@@ -941,7 +961,7 @@ def test_position():
               maxinterval=0, desc='pos1 bar', position=1)
     t3 = tqdm(total=2, file=our_file, miniters=1, mininterval=0,
               maxinterval=0, desc='pos2 bar', position=2)
-    for i in _range(2):
+    for _ in _range(2):
         t1.update()
         t3.update()
         t2.update()
@@ -1025,13 +1045,13 @@ def test_deprecated_gui():
         assert not hasattr(t, "sp")
         try:
             t.update(1)
-        except DeprecationWarning as e:
-            if str(e) != ('Please use tqdm_gui(...) instead of'
-                          ' tqdm(..., gui=True)'):
+        except TqdmDeprecationWarning as e:
+            if 'Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`' \
+                    not in our_file.getvalue():
                 raise
         else:
-            raise DeprecationWarning('Should not allow manual gui=True without'
-                                     ' overriding __iter__() and update()')
+            raise DeprecationError('Should not allow manual gui=True without'
+                                   ' overriding __iter__() and update()')
         finally:
             t._instances.clear()
             # t.close()
@@ -1042,13 +1062,13 @@ def test_deprecated_gui():
         try:
             for _ in t:
                 pass
-        except DeprecationWarning as e:
-            if str(e) != ('Please use tqdm_gui(...) instead of'
-                          ' tqdm(..., gui=True)'):
+        except TqdmDeprecationWarning as e:
+            if 'Please use `tqdm_gui(...)` instead of `tqdm(..., gui=True)`' \
+                    not in our_file.getvalue():
                 raise e
         else:
-            raise DeprecationWarning('Should not allow manual gui=True without'
-                                     ' overriding __iter__() and update()')
+            raise DeprecationError('Should not allow manual gui=True without'
+                                   ' overriding __iter__() and update()')
         finally:
             t._instances.clear()
             # t.close()
@@ -1113,6 +1133,16 @@ def test_clear():
         assert after == ['', '']
 
 
+@with_setup(pretest, posttest)
+def test_clear_disabled():
+    """ Test clearing bar display """
+    with closing(StringIO()) as our_file:
+        with tqdm(total=10, file=our_file, desc='pos0 bar', disable=True,
+                  bar_format='{l_bar}') as t:
+            t.clear()
+        assert our_file.getvalue() == ''
+
+
 @with_setup(pretest, posttest)
 def test_refresh():
     """ Test refresh bar display """
@@ -1135,6 +1165,18 @@ def test_refresh():
         assert after == [u'pos0 bar:  10%|', u'pos1 bar:  10%|']
 
 
+@with_setup(pretest, posttest)
+def test_disabled_refresh():
+    """ Test refresh bar display """
+    with closing(StringIO()) as our_file:
+        with tqdm(total=10, file=our_file, desc='pos0 bar', disable=True,
+                  bar_format='{l_bar}', mininterval=999, miniters=999) as t:
+            t.update()
+            t.refresh()
+
+        assert our_file.getvalue() == ''
+
+
 @with_setup(pretest, posttest)
 def test_write():
     """ Test write messages """
@@ -1228,3 +1270,149 @@ def test_len():
     with closing(StringIO()) as f:
         with tqdm(np.zeros((3, 4)), file=f) as t:
             assert len(t) == 3
+
+
+@with_setup(pretest, posttest)
+def test_autodisable_disable():
+    """Test autodisable will disable on non-TTY"""
+    with closing(StringIO()) as our_file:
+        with tqdm(total=10, disable=None, file=our_file) as t:
+            t.update(3)
+        assert our_file.getvalue() == ''
+
+
+@with_setup(pretest, posttest)
+def test_autodisable_enable():
+    """Test autodisable will not disable on TTY"""
+    with closing(StringIO()) as our_file:
+        setattr(our_file, "isatty", lambda: True)
+        with tqdm(total=10, disable=None, file=our_file) as t:
+            t.update()
+        assert our_file.getvalue() != ''
+
+
+@with_setup(pretest, posttest)
+def test_deprecation_exception():
+    def test_TqdmDeprecationWarning():
+        with closing(StringIO()) as our_file:
+            raise (TqdmDeprecationWarning('Test!',
+                                          fp_write=getattr(our_file, 'write',
+                                                           sys.stderr.write)))
+
+    def test_TqdmDeprecationWarning_nofpwrite():
+        raise (TqdmDeprecationWarning('Test!', fp_write=None))
+
+    assert_raises(TqdmDeprecationWarning, test_TqdmDeprecationWarning)
+    assert_raises(Exception, test_TqdmDeprecationWarning_nofpwrite)
+
+
+@with_setup(pretest, posttest)
+def test_monitoring_thread():
+    # Note: should fix miniters for these tests, else with dynamic_miniters
+    # it's too complicated to handle with monitoring update and maxinterval...
+    maxinterval = 10
+
+    # 1- Configure and test the thread alone
+    # Setup a discrete timer
+    timer = DiscreteTimer()
+    TMonitor._time = timer.time
+    # And a fake sleeper
+    sleeper = FakeSleep(timer)
+    TMonitor._sleep = sleeper.sleep
+
+    # And a fake tqdm
+    class fake_tqdm(object):
+        _instances = []
+
+    # Instanciate the monitor
+    monitor = TMonitor(fake_tqdm, maxinterval)
+    # Test if alive, then killed
+    assert monitor.report()
+    monitor.exit()
+    timer.sleep(maxinterval*2)  # need to go out of the sleep to die
+    assert not monitor.report()
+    # assert not monitor.is_alive()  # not working dunno why, thread not killed
+    del monitor
+
+    # 2- Test for real with a tqdm instance that takes too long
+    total = 1000
+    # Setup a discrete timer
+    timer = DiscreteTimer()
+    # And a fake sleeper
+    sleeper = FakeSleep(timer)
+    # Setup TMonitor to use the timer
+    TMonitor._time = timer.time
+    TMonitor._sleep = sleeper.sleep
+    # Set monitor interval
+    tqdm.monitor_interval = maxinterval
+    with closing(StringIO()) as our_file:
+        with tqdm(total=total, file=our_file, miniters=500,
+                  mininterval=0.1, maxinterval=maxinterval) as t:
+            cpu_timify(t, timer)
+            # Do a lot of iterations in a small timeframe
+            # (smaller than monitor interval)
+            timer.sleep(maxinterval/2)  # monitor won't wake up
+            t.update(500)
+            # check that our fixed miniters is still there
+            assert t.miniters == 500
+            # Then do 1 it after monitor interval, so that monitor kicks in
+            timer.sleep(maxinterval*2)
+            t.update(1)
+            # Wait for the monitor to get out of sleep's loop and update tqdm..
+            timeend = timer.time()
+            while not (t.monitor.woken >= timeend and t.miniters == 1):
+                timer.sleep(1)  # Force monitor to wake up if it woken too soon
+                sleep(0.000001)  # sleep to allow interrupt (instead of pass)
+            assert t.miniters == 1  # check that monitor corrected miniters
+            # Note: at this point, there may be a race condition: monitor saved
+            # current woken time but timer.sleep() happen just before monitor
+            # sleep. To fix that, either sleep here or increase time in a loop
+            # to ensure that monitor wakes up at some point.
+
+            # Try again but already at miniters = 1 so nothing will be done
+            timer.sleep(maxinterval*2)
+            t.update(2)
+            timeend = timer.time()
+            while not (t.monitor.woken >= timeend):
+                timer.sleep(1)  # Force monitor to wake up if it woken too soon
+                sleep(0.000001)
+            # Wait for the monitor to get out of sleep's loop and update tqdm..
+            assert t.miniters == 1  # check that monitor corrected miniters
+
+    # 3- Check that class var monitor is deleted if no instance left
+    assert tqdm.monitor is None
+
+    # 4- Test on multiple bars, one not needing miniters adjustment
+    total = 1000
+    # Setup a discrete timer
+    timer = DiscreteTimer()
+    # And a fake sleeper
+    sleeper = FakeSleep(timer)
+    # Setup TMonitor to use the timer
+    TMonitor._time = timer.time
+    TMonitor._sleep = sleeper.sleep
+    with closing(StringIO()) as our_file:
+        with tqdm(total=total, file=our_file, miniters=500,
+                  mininterval=0.1, maxinterval=maxinterval) as t1:
+            # Set high maxinterval for t2 so monitor does not need to adjust it
+            with tqdm(total=total, file=our_file, miniters=500,
+                      mininterval=0.1, maxinterval=1E5) as t2:
+                cpu_timify(t1, timer)
+                cpu_timify(t2, timer)
+                # Do a lot of iterations in a small timeframe
+                timer.sleep(5)
+                t1.update(500)
+                t2.update(500)
+                assert t1.miniters == 500
+                assert t2.miniters == 500
+                # Then do 1 it after monitor interval, so that monitor kicks in
+                timer.sleep(maxinterval*2)
+                t1.update(1)
+                t2.update(1)
+                # Wait for the monitor to get out of sleep and update tqdm
+                timeend = timer.time()
+                while not (t.monitor.woken >= timeend and t1.miniters == 1):
+                    timer.sleep(1)
+                    sleep(0.000001)
+                assert t1.miniters == 1  # check that monitor corrected miniters
+                assert t2.miniters == 500  # check that t2 was not adjusted
